## TODO - Fix Weekday situation
from elasticsearch import Elasticsearch, helpers
from time import strftime, localtime
import contextlib
import sys

class RecycleObject(object):
    def write(self, junk): pass
    def flush(self): pass

@contextlib.contextmanager
def nostdout():
    savestdout = sys.stdout
    sys.stdout = RecycleObject()
    yield
    sys.stdout = savestdout

def weekday(day):
    weekdays = {
            "Monday": 1,
            "Tuesday": 2,
            "Wednesday": 3,
            "Thursday": 4,
            "Friday": 5,
            "Saturday": 6,
            "Sunday": 7,
            }

    return weekdays[day]

def hour(datetime):
    return strftime("%H", localtime(datetime))

def Tweet(Tweet, config):
    weekdays = {
            "Monday": 1,
            "Tuesday": 2,
            "Wednesday": 3,
            "Thursday": 4,
            "Friday": 5,
            "Saturday": 6,
            "Sunday": 7,
            }
    day = weekdays[strftime("%A", localtime(Tweet.datetime))]

    actions = []
    nLikes = 0
    nReplies = 0
    nRetweets = 0

    dt = f"{Tweet.datestamp} {Tweet.timestamp}"

    j_data = {
            "_index": config.Index_tweets,
            "_type": "items",
            "_id": Tweet.id + "_raw_" + config.Essid,
            "_source": {
                "id": Tweet.id,
                "date": dt,
                "timezone": Tweet.timezone,
                "location": Tweet.location,
                "tweet": Tweet.tweet,
                "hashtags": Tweet.hashtags,
                "user_id": Tweet.user_id,
                "username": Tweet.username,
                "day": day,
                "hour": hour(Tweet.datetime),
                "link": Tweet.link,
                "retweet": Tweet.retweet,
                "user_rt": Tweet.user_rt,
                "essid": config.Essid
                }
            }
    actions.append(j_data)

    for l in range(int(Tweet.likes)):
        j_data = {
                "_index": config.Index_tweets,
                "_type": "items",
                "_id": Tweet.id + "_likes_" + str(nLikes) + "_" + config.Essid,
                "_source": {
                    "id": Tweet.id,
                    "date": dt,
                    "timezone": Tweet.timezone,
                    "location": Tweet.location,
                    "tweet": Tweet.tweet,
                    "hashtags": Tweet.hashtags,
                    "likes": True,
                    "user_id": Tweet.user_id,
                    "username": Tweet.username,
                    "day": day,
                    "hour": hour(Tweet.datetime),
                    "link": Tweet.link,
                    "retweet": Tweet.retweet,
                    "user_rt": Tweet.user_rt,
                    "essid": config.Essid
                    }
                }
        actions.append(j_data)
        nLikes += 1

    for rep in range(int(Tweet.replies)):
        j_data = {
                "_index": config.Index_tweets,
                "_type": "items",
                "_id": Tweet.id + "_replies_" + str(nReplies) + "_" + config.Essid,
                "_source": {
                    "id": Tweet.id,
                    "date": dt,
                    "timezone": Tweet.timezone,
                    "location": Tweet.location,
                    "tweet": Tweet.tweet,
                    "hashtags": Tweet.hashtags,
                    "replies": True,
                    "user_id": Tweet.user_id,
                    "username": Tweet.username,
                    "day": day,
                    "hour": hour(Tweet.datetime),
                    "link": Tweet.link,
                    "retweet": Tweet.retweet,
                    "user_rt": Tweet.user_rt,
                    "essid": config.Essid
                    }
                }
        actions.append(j_data)
        nReplies += 1

    for ret in range(int(Tweet.retweets)):
        j_data = {
                "_index": config.Index_tweets,
                "_type": "items",
                "_id": Tweet.id + "_retweets_" + str(nRetweets) + "_" + config.Essid,
                "_source": {
                    "id": Tweet.id,
                    "date": dt,
                    "timezone": Tweet.timezone,
                    "location": Tweet.location,
                    "tweet": Tweet.tweet,
                    "hashtags": Tweet.hashtags,
                    "retweets": True,
                    "user_id": Tweet.user_id,
                    "username": Tweet.username,
                    "day": day,
                    "hour": hour(Tweet.datetime),
                    "link": Tweet.link,
                    "retweet": Tweet.retweet,
                    "user_rt": Tweet.user_rt,
                    "essid": config.Essid
                    }
                }
        actions.append(j_data)
        nRetweets += 1

    es = Elasticsearch(config.Elasticsearch)
    with nostdout():
        helpers.bulk(es, actions, chunk_size=2000, request_timeout=200)
    actions = []

def Follow(user, config):
    actions = []

    j_data = {
            "_index": config.Index_follow,
            "_type": "items",
            "_id": user + "_" + config.Username + "_" + config.Essid,
            "_source": {
                "user": user,
                "follow": config.Username,
                "essid": config.Essid
                }
            }
    actions.append(j_data)

    es = Elasticsearch(config.Elasticsearch)
    with nostdout():
        helpers.bulk(es, actions, chunk_size=2000, request_timeout=200)
    actions = []

def UserProfile(user, config):
    actions = []

    j_data = {
            "_index": config.Index_users,
            "_type": "items",
            "_id": user.id + "_" + user.join_date + "_" + user.join_time + "_" + config.Essid,
            "_source": {
                "id": user.id,
                "name": user.name,
                "username": user.username,
                "bio": user.bio,
                "location": user.location,
                "url": user.url,
                "join_datetime": user.join_date + " " + user.join_time,
                "join_date": user.join_date,
                "join_time": user.join_time,
                "tweets": user.tweets,
                "following": user.following,
                "followers": user.followers,
                "likes": user.likes,
                "media": user.media_count,
                "private": user.is_private,
                "verified": user.is_verified,
                "avatar": user.avatar,
                "session": config.Essid
                }
            }
    actions.append(j_data)

    es = Elasticsearch(config.Elasticsearch)
    with nostdout():
        helpers.bulk(es, actions, chunk_size=2000, request_timeout=200)
    actions = []



from . import format
from .tweet import Tweet
from .user import User
from datetime import datetime
from .storage import db, elasticsearch, write, panda

tweets_object = []

def datecheck(datestamp, config):
    if config.Since and config.Until:
        d = int(datestamp.replace("-", ""))
        s = int(config.Since.replace("-", ""))
        if d < s:
            return False
    return True

def is_tweet(tw):
    try:
        tw.find("div")["data-item-id"]
        return True
    except:
        return False

def _output(obj, output, config):
    if config.Output != None:
        if config.Store_csv:
            write.Csv(obj, config)
        elif config.Store_json:
            write.Json(obj, config)
        else:
            write.Text(output, config.Output)

    if config.Pandas:
        Pandas.update(obj, config.Essid)
    if config.Elasticsearch:
        if config.Store_object:
            tweets_object.append(obj)
        else:
            print(output, end=".", flush=True)
    else:
        if config.Store_object:
            tweets_object.append(obj)
        else:
            try:
                print(output)
            except UnicodeEncodeError:
                pass

async def Tweets(tw, location, config, conn):
    copyright = tw.find("div", "StreamItemContent--withheld")
    if copyright is None and is_tweet(tw):
        tweet = Tweet(tw, location, config)
        if datecheck(tweet.datestamp, config):
            output = format.Tweet(config, tweet)
            
            if config.Database:
                db.tweets(conn, tweet, config)
            
            if config.Elasticsearch:
                elasticsearch.Tweet(tweet, config)
            
            _output(tweet, output, config)

async def Users(u, config, conn):
    user = User(u)
    output = format.User(config.Format, user)
    
    if config.Database:
        db.user(conn, config.Username, config.Followers, user)

    if config.Elasticsearch:
        _save_date =  user.join_date
        _save_time = user.join_time
        user.join_date = str(datetime.strptime(user.join_date, "%d %b %Y")).split()[0]
        user.join_time = str(datetime.strptime(user.join_time, "%I:%M %p")).split()[1]
        elasticsearch.UserProfile(user, config)
        user.join_date = _save_date
        user.join_time = _save_time

    _output(user, output, config)

async def Username(username, config, conn):
    if config.Database:
        db.follow(conn, config.Username, config.Followers, username)

    if config.Elasticsearch:
        elasticsearch.Follow(username, config)

    _output(username, username, config)



class user:
    pass

def inf(ur, _type):
    try:
        group = ur.find("div", "user-actions btn-group not-following ")
    except:
        group = ur.find("div", "user-actions btn-group not-following protected")
    
    if _type == "id":
        ret = group["data-user-id"]
    elif _type == "name":
        ret = group["data-name"]
    elif _type == "username":
        ret = group["data-screen-name"]
    elif _type == "private":
        ret = group["data-protected"]
    
    return ret

def card(ur, _type):
    if _type == "bio":
        try:
            ret = ur.find("p", "ProfileHeaderCard-bio u-dir").text.replace("\n", " ")
        except:
            ret = "None"
    elif _type == "location":
        try:
            ret = ur.find("span", "ProfileHeaderCard-locationText u-dir").text
            ret = ret[15:].replace("\n", " ")[:-10]
        except:
            ret = "None"
    elif _type == "url":
        try:
            ret = ur.find("span", "ProfileHeaderCard-urlText u-dir").find("a")["title"]
        except:
            ret = "None"
    
    return ret

def join(ur):
    jd = ur.find("span", "ProfileHeaderCard-joinDateText js-tooltip u-dir")["title"]
    return jd.split(" - ")

def stat(ur, _type):
    _class = f"ProfileNav-item ProfileNav-item--{_type}"
    stat = ur.find("li", _class)
    return stat.find("span", "ProfileNav-value")["data-count"]

def media(ur):
    try:
        media_count = ur.find("a", "PhotoRail-headingWithCount js-nav").text
        media_count = media_count.replace("\n", "")[32:].split(" ")[0]
    except:
        media_count = "0"

    return media_count

def verified(ur):
    try:
        is_verified = ur.find("span", "ProfileHeaderCard-badges").text
        if "Verified account" in is_verified:
            is_verified = "true"
        else:
            is_verified = "false"
    except:
        is_verified = "false"

    return is_verified

def User(ur):
    u = user()
    for img in ur.findAll("img", "Emoji Emoji--forText"):
        img.replaceWith(img["alt"])
    u.id = inf(ur, "id")
    u.name = inf(ur, "name")
    u.username = inf(ur, "username")
    u.bio = card(ur, "bio")
    u.location = card(ur, "location")
    u.url = card(ur, "url")
    u.join_date = join(ur)[1]
    u.join_time = join(ur)[0]
    u.tweets = stat(ur, "tweets is-active")
    u.following = stat(ur, "following")
    u.followers = stat(ur, "followers")
    u.likes = stat(ur, "favorites")
    u.media_count = media(ur)
    u.is_private = inf(ur, "private")
    u.is_verified = verified(ur)
    u.avatar = ur.find("img", "ProfileAvatar-image")["src"]
    return u



def Tweet(config, t):
    if config.Format:
        output = config.Format.replace("{id}", t.id)
        output = output.replace("{date}", t.datestamp)
        output = output.replace("{time}", t.timestamp)
        output = output.replace("{user_id}", t.user_id)
        output = output.replace("{username}", t.username)
        output = output.replace("{timezone}", t.timezone)
        output = output.replace("{tweet}", t.tweet)
        output = output.replace("{location}", t.location)
        output = output.replace("{hashtags}", str(t.hashtags))
        output = output.replace("{replies}", t.replies)
        output = output.replace("{retweets}", t.retweets)
        output = output.replace("{likes}", t.likes)
        output = output.replace("{link}", t.link)
        output = output.replace("{is_retweet}", str(t.retweet))
        output = output.replace("{user_rt}", t.user_rt)
        output = output.replace("{mentions}", str(t.mentions))
    else:
        output = f"{t.id} {t.datestamp} {t.timestamp} {t.timezone} "

        if config.Profile and t.username.lower() != config.Username:
           output += "RT "

        output += f"<{t.username}> {t.tweet}"

        if config.Show_hashtags:
            hashtags = ",".join(t.hashtags)
            output += f" {hashtags}"
        if config.Stats:
            output += f" | {t.replies} replies {t.retweets} retweets {t.likes} likes"
        if config.Location:
            output += f" | Location {t.location}"

    return output

def User(_format, u):
    if _format:
        output = _format.replace("{id}", u.id)
        output += output.replace("{name}", u.name)
        output += output.replace("{username}", u.username)
        output += output.replace("{bio}", u.bio)
        output += output.replace("{location}", u.location)
        output += output.replace("{url}", u.url)
        output += output.replace("{join_date}", u.join_date)
        output += output.replace("{join_time}", u.join_time)
        output += output.replace("{tweets}", u.tweets)
        output += output.replace("{following}", u.following)
        output += output.replace("{followers}", u.followers)
        output += output.replace("{likes}", u.likes)
        output += output.replace("{media}", u.media_count)
        output += output.replace("{private}", u.is_private)
        output += output.replace("{verified}", u.is_verified)
        output += output.replace("{avatar}", u.avatar)
    else:
        output = f"{u.id} | {u.name} | @{u.username} | Private: "
        output += f"{u.is_private} | Verified: {u.is_verified} |"
        output += f" Bio: {u.bio} | Location: {u.location} | Url: "
        output += f"{u.url} | Joined: {u.join_date} {u.join_time} "
        output += f"| Tweets: {u.tweets} | Following: {u.following}"
        output += f" | Followers: {u.followers} | Likes: {u.likes} "
        output += f"| Media: {u.media_count} | Avatar: {u.avatar}"

    return output



from .elasticsearch import *
from time import strftime, localtime
import pandas as pd
import warnings

_blocks = []

def update(Tweet, session):
    day = weekday(strftime("%A", localtime(Tweet.datetime)))
    dt = f"{Tweet.datestamp} {Tweet.timestamp}"

    _data = {
                "id": Tweet.id,
                "date": dt,
                "timezone": Tweet.timezone,
                "location": Tweet.location,
                "tweet": Tweet.tweet,
                "hashtags": Tweet.hashtags,
                "user_id": Tweet.user_id,
                "username": Tweet.username,
                "day": day,
                "hour": hour(Tweet.datetime),
                "link": Tweet.link,
                "retweet": Tweet.retweet,
                "user_rt": Tweet.user_rt,
                "essid": str(session),
                'mentions': Tweet.mentions
                }
    _blocks.append(_data)

def get():
    df = pd.DataFrame(_blocks)
    return df

def save(_filename, _dataframe, **options):
    if options.get("dataname"):
        _dataname = options.get("dataname")
    else:
        _dataname = "twint"

    if not options.get("type"):
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            _store = pd.HDFStore(_filename)
            _store[_dataname] = _dataframe
            _store.close()
    elif options.get("type") == "Pickle":
        with warnings.catch_warnings():
            warnings.simplefilter("ignore")
            _dataframe.to_pickle(_filename)
    else:
        print("Please specify: filename, DataFrame, DataFrame name and type (HDF5, default, or Pickle")

def read(_filename, **options):
    if not options.get("dataname"):
        _dataname = "Twint"

    if not options.get("type"):
        _store = pd.HDFStore(_filename)
        df = _store[_dataname]
        return df
    elif options.get("type") == "Pickle":
        df = pd.read_pickle(_filename)
        return df
    else:
        print("Please specify: DataFrame, DataFrame name (twint as default), filename and type (HDF5, default, or Pickle")



mobile = "https://mobile.twitter.com"
base = "https://twitter.com/i"

async def Favorites(username, init):
    url = f"{mobile}/{username}/favorites?lang=en"

    if init != -1:
        url += f"&max_id={init}"

    return url

async def Followers(username, init):
    url = f"{mobile}/{username}/followers?lang=en"

    if init != -1:
        url += f"&cursor={init}"

    return url

async def Following(username, init):
    url = f"{mobile}/{username}/following?lang=en"

    if init != -1:
        url += f"&cursor={init}"

    return url

async def MobileProfile(username, init):
    url = f"{mobile}/{username}?lang=en"

    if init != -1:
        url += f"&max_id={init}"

    return url

async def Profile(username, init):
    url = f"{base}/profiles/show/{username}/timeline/tweets?include_"
    url += "available_features=1&lang=en&include_entities=1"
    url += "&include_new_items_bar=true"

    if init != -1:
        url += f"&max_position={init}"

    return url

async def Search(config, init):
    url = f"{base}/search/timeline?f=tweets&vertical=default&lang=en"
    url += "&include_available_features=1&include_entities=1&"
    url += f"reset_error_state=false&src=typd&qf=off&max_position={init}&q="

    if config.Lang:
        url = url.replace("lang=en", f"l={config.Lang}&lang=en")
    if config.Username:
        url += f"from%3A{config.Username}"
    if config.Geo:
        config.Geo = config.Geo.replace(" ", "")
        url += f"geocode%3A{config.Geo}"
    if config.Search:
        config.Search = config.Search.replace(" ", "%20")
        config.Search = config.Search.replace("#", "%23")
        url += f"%20{config.Search}"
    if config.Year:
        url += f"%20until%3A{config.Year}-1-1"
    if config.Since:
        url += f"%20since%3A{config.Since}"
    if config.Until:
        url += f"%20until%3A{config.Until}"
    if config.Fruit:
        url += "%20myspace.com%20OR%20last.fm%20OR"
        url += "%20mail%20OR%20email%20OR%20gmail%20OR%20e-mail"
        url += "%20OR%20phone%20OR%20call%20me%20OR%20text%20me"
        url += "%20OR%20keybase"
    if config.Verified:
        url += "%20filter%3Averified"
    if config.To:
        url += f"%20to%3A{config.To}"
    if config.All:
        url += f"%20to%3A{config.All}%20OR%20from%3A{config.All}%20OR%20@{config.All}"
    if config.Near:
        config.Near = config.Near.replace(" ", "%20")
        config.Near = config.Near.replace(",", "%2C")
        url += f"%20near%3A%22{config.Near}%22"
    if config.Images:
        url += "%20filter%3Aimages"
    if config.Videos:
        url += "%20filter%3Avideos"
    if config.Media:
        url += "%20filter%3Amedia"
    if config.Replies:
        url += "%20filter%3Areplies"

    return url



import datetime

class Datelock:
    _until = None
    _since = None
    _since_def_user = None

def Set(Until, Since):
    d = Datelock()

    if Until:
        d._until = datetime.datetime.strptime(Until, "%Y-%m-%d").date()
    else:
        d._until = datetime.date.today()

    if Since:
        d._since = datetime.datetime.strptime(Since, "%Y-%m-%d").date()
        d._since_def_user = True
    else:
        d._since = datetime.datetime.strptime("2006-03-21", "%Y-%m-%d").date()
        d._since_def_user = False

    return d



from time import strftime, localtime
import re

class tweet:
    pass

def getMentions(tw):
    try:
        mentions = tw.find("div", "js-original-tweet")["data-mentions"].split(" ")
    except:
        mentions = ""

    return mentions

def getText(tw):
    text = tw.find("p", "tweet-text").text
    text = text.replace("\n", "")
    text = text.replace("http", " http")
    text = text.replace("pic.twitter", " pic.twitter")

    return text

def getTweet(tw, mentions):
    try:
        text = getText(tw)
        for i in range(len(mentions)):
            mention = f"@{mentions[i]}"
            if mention not in text:
                text = f"{mention} {text}"
    except:
        text = getText(tw)

    return text

def getHashtags(text):
    return re.findall(r'(?i)\#\w+', text, flags=re.UNICODE)

def getStat(tw, _type):
    st = f"ProfileTweet-action--{_type} u-hiddenVisually"
    return tw.find("span", st).find("span")["data-tweet-stat-count"]

def getRetweet(profile, username, user):
    if profile and username.lower() != user:
        return True

def getUser_rt(profile, username, user):
    if getRetweet(profile, username, user):
        user_rt = user
    else:
        user_rt = "None"
    
    return user_rt

def Tweet(tw, location, config):
    t = tweet()
    t.id = tw.find("div")["data-item-id"]
    t.datetime = int(tw.find("span", "_timestamp")["data-time"])
    t.datestamp = strftime("%Y-%m-%d", localtime(t.datetime))
    t.timestamp = strftime("%H:%M:%S", localtime(t.datetime))
    t.user_id = tw.find("a", "account-group js-account-group js-action-profile js-user-profile-link js-nav")["data-user-id"]
    t.username = tw.find("span", "username").text.replace("@", "")
    t.timezone = strftime("%Z", localtime())
    for img in tw.findAll("img", "Emoji Emoji--forText"):
        img.replaceWith(img["alt"])
    t.mentions = getMentions(tw)
    t.tweet = getTweet(tw, t.mentions)
    t.location = location
    t.hashtags = getHashtags(t.tweet)
    t.replies = getStat(tw, "reply")
    t.retweets = getStat(tw, "retweet")
    t.likes = getStat(tw, "favorite")
    t.link = f"https://twitter.com/{t.username}/status/{t.id}"
    t.retweet = getRetweet(config.Profile, t.username, config.Username)
    t.user_rt = getUser_rt(config.Profile, t.username, config.Username)
    return t



class Config:
    Username = None
    User_id = None
    Search = None
    Geo = None
    Location = False
    Near = None
    Lang = None
    Output = None
    Elasticsearch = None
    Timedelta = None
    Year = None
    Since = None
    Until = None
    Fruit = False
    Verified = False
    Store_csv = False
    Store_json = False
    Custom = False
    Show_hashtags = False
    Limit = None
    Count = None
    Stats = False
    Database = None
    To = None
    All = None
    Debug = False
    Format = None
    Essid = ""
    Profile = False
    Followers = False
    Following = False
    Favorites = False
    TwitterSearch = False
    User_full = False
    Profile_full = False
    Store_object = False
    Store_pandas = False
    Pandas_type = None
    Pandas = False
    Index_tweets = "twint"
    Index_follow = "twintGraph"
    Index_users = "twintUser"
    Debug = False
    Resume = None
    Images = False
    Videos = False
    Media = False
    Replies = False



from . import url
from .output import Tweets, Users
from async_timeout import timeout
from bs4 import BeautifulSoup
import aiohttp
import asyncio
import concurrent.futures

async def RequestUrl(config, init):
    if config.Profile:
        if config.Profile_full:
            _url = await url.MobileProfile(config.Username, init)
            response = await MobileRequest(_url)
        else:
            _url = await url.Profile(config.Username, init)
            response = await Request(_url)
    elif config.TwitterSearch:
        _url = await url.Search(config, init)
        response = await Request(_url)
    else:
        if config.Following:
            _url = await url.Following(config.Username, init)
        elif config.Followers:
            _url = await url.Followers(config.Username, init)
        else:
            _url = await url.Favorites(config.Username, init)
        response = await MobileRequest(_url)

    if config.Debug:
        print(_url, file=open("twint-request_urls.log", "a", encoding="utf-8"))

    return response

async def MobileRequest(url):
    ua = {'User-Agent': 'Lynx/2.8.5rel.1 libwww-FM/2.14 SSL-MM/1.4.1 GNUTLS/0.8.12'}
    connect = aiohttp.TCPConnector(verify_ssl=False)
    async with aiohttp.ClientSession(headers=ua, connector=connect) as session:
        return await Response(session, url)

async def Request(url):
    connect = aiohttp.TCPConnector(verify_ssl=False)
    async with aiohttp.ClientSession(connector=connect) as session:
        return await Response(session, url)

async def Response(session, url):
    with timeout(30):
        async with session.get(url) as response:
            return await response.text()

async def Username(_id):
    url = f"https://twitter.com/intent/user?user_id={_id}&lang=en"
    r = Request(url)
    soup = BeautifulSoup(r, "html.parser")

    return soup.find("a", "fn url alternate-context")["href"].replace("/", "")

async def Tweet(url, config, conn):
    try:
        response = await Request(url)
        soup = BeautifulSoup(response, "html.parser")
        tweet = soup.find("div", "permalink-inner permalink-tweet-container")
        location = soup.find("span", "ProfileHeaderCard-locationText u-dir").text
        location = location[15:].replace("\n", " ")[:-10]
        await Tweets(tweet, location, config, conn)
    except:
        pass

async def User(url, config, conn):
    try:
        response = await Request(url)
        soup = BeautifulSoup(response, "html.parser")
        await Users(soup, config, conn)
    except:
        pass

def Limit(Limit, count):
    if Limit is not None and count >= int(Limit):
        return True

async def Multi(feed, config, conn):
    count = 0
    try:
        with concurrent.futures.ThreadPoolExecutor(max_workers=20) as executor:
            loop = asyncio.get_event_loop()
            futures = []
            for tweet in feed:
                count += 1
                if config.Favorites or config.Profile_full:
                    link = tweet.find("a")["href"]
                    url = f"https://twitter.com{link}&lang=en"
                elif config.User_full:
                    username = tweet.find("a")["name"]
                    url = f"http://twitter.com/{username}?lang=en"
                else:
                    link = tweet.find("a", "tweet-timestamp js-permalink js-nav js-tooltip")["href"]
                    url = f"https://twitter.com{link}?lang=en"
                
                if config.User_full:
                    futures.append(loop.run_in_executor(executor, await User(url,
                        config, conn)))
                else:
                    futures.append(loop.run_in_executor(executor, await Tweet(url,
                        config, conn)))

            await asyncio.gather(*futures)
    except:
        pass

    return count



'''
TWINT - Twitter Intelligence Tool (formerly known as Tweep).

See wiki on Github for in-depth details.
https://github.com/haccer/twint/wiki

Licensed under MIT License
Copyright (c) 2018 Cody Zacharias
'''
from .config import Config
from . import run



#!/usr/bin/python3
from setuptools import setup
import io
import os
import sys

# Package meta-data
NAME = 'twint'
DESCRIPTION = 'An advanced Twitter scraping & OSINT tool.'
URL = 'https://github.com/twintproject/twint'
EMAIL = 'codyzacharias@pm.me'
AUTHOR = 'Cody Zacharias'
REQUIRES_PYTHON = '>=3.5.0'
VERSION = None

# Packages required
REQUIRED = [
		'aiohttp', 'aiodns', 'beautifulsoup4', 'cchardet', 
                'elasticsearch', 'pysocks', 'pandas'
		]

here = os.path.abspath(os.path.dirname(__file__))

with io.open(os.path.join(here, 'README.md'), encoding='utf-8') as f:
	long_description = '\n' + f.read()

# Load the package's __version__.py
about = {}
if not VERSION:
	with open(os.path.join(here, NAME, '__version__.py')) as f:
		exec(f.read(), about)
else:
	about['__version__'] = VERSION

setup(
	name=NAME,
	version=about['__version__'],
	description=DESCRIPTION,
	long_description=long_description,
	author=AUTHOR,
	author_email=EMAIL,
	python_requires=REQUIRES_PYTHON,
	url=URL,
	packages=['twint', 'twint.storage'],
	install_requires=REQUIRED,
	license='MIT',
	classifiers=[
		'License :: OSI Approved :: MIT License',
		'Programming Language :: Python',
		'Programming Language :: Python :: 3',
		'Programming Language :: Python :: 3.5',
		'Programming Language :: Python :: 3.6'
		],
)



from . import write_meta as meta
import csv
import json
import os

def Text(entry, f):
    print(entry, file=open(f, "a", encoding="utf-8"))

def Type(config):
    if config.User_full:
        _type = "user"
    elif config.Followers or config.Following:
        _type = "username"
    else:
        _type = "tweet"

    return _type

def struct(obj, custom, _type):
    if custom:
        fieldnames = custom
        row = {}
        for f in fieldnames:
            row[f] = meta.Data(obj, _type)[f]
    else:
        fieldnames = meta.Fieldnames(_type)
        row = meta.Data(obj, _type)

    return fieldnames, row

def Csv(obj, config):
    fieldnames, row = struct(obj, config.Custom, Type(config))

    if not (os.path.exists(config.Output)):
        with open(config.Output, "w", newline='', encoding="utf-8") as csv_file:
            writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
            writer.writeheader()

    with open(config.Output, "a", newline='', encoding="utf-8") as csv_file:
        writer = csv.DictWriter(csv_file, fieldnames=fieldnames)
        writer.writerow(row)

def Json(obj, config):
    null, data = struct(obj, config.Custom, Type(config))

    with open(config.Output, "a", newline='', encoding="utf-8") as json_file:
        json.dump(data, json_file)
        json_file.write("\n")



from . import datelock, feed, get, output, verbose
from asyncio import get_event_loop
from datetime import timedelta
from .storage import db

class Twint:
    def __init__(self, config):
        if config.Resume is not None and config.TwitterSearch:
            self.init = f"TWEET-{config.Resume}-0"
        else:
            self.init = -1
        self.feed = [-1]
        self.count = 0
        self.config = config
        self.conn = db.Conn(config.Database)
        self.d = datelock.Set(self.config.Until, self.config.Since)
        verbose.Elastic(config.Elasticsearch)

        if not self.config.Timedelta:
            if (self.d._until - self.d._since).days > 30:
                self.config.Timedelta = 30
            else:
                self.config.Timedelta = (self.d._until - self.d._since).days

    async def Feed(self):
        response = await get.RequestUrl(self.config, self.init)
        if self.config.Debug:
            print(response, file=open("twint-last-request.log", "w", encoding="utf-8"))

        self.feed = []
        try:
            if self.config.Favorites:
                self.feed, self.init = feed.Mobile(response)
            elif self.config.Followers or self.config.Following:
                self.feed, self.init = feed.Follow(response)
            elif self.config.Profile:
                if self.config.Profile_full:
                    self.feed, self.init = feed.Mobile(response)
                else:
                    self.feed, self.init = feed.profile(response)
            elif self.config.TwitterSearch:
                self.feed, self.init = feed.Json(response)
        except:
            pass

    async def follow(self):
        await self.Feed()
        if self.config.User_full:
            self.count += await get.Multi(self.feed, self.config, self.conn)
        else:
            for user in self.feed:
                self.count += 1
                username = user.find("a")["name"]
                await output.Username(username, self.config, self.conn)

    async def favorite(self):
        await self.Feed()
        self.count += await get.Multi(self.feed, self.config, self.conn)

    async def profile(self):
        await self.Feed()
        if self.config.Profile_full:
            self.count += await get.Multi(self.feed, self.config, self.conn)
        else:
            for tweet in self.feed:
                self.count += 1
                await output.Tweets(tweet, "", self.config, self.conn)

    async def tweets(self):
        await self.Feed()
        if self.config.Location:
            self.count += await get.Multi(self.feed, self.config, self.conn)
        else:
            for tweet in self.feed:
                self.count += 1
                await output.Tweets(tweet, "", self.config, self.conn)

    async def main(self):
        if self.config.User_id is not None:
            self.config.Username = await get.Username(self.config.User_id)
        
        if self.config.TwitterSearch and self.config.Since and self.config.Until:
            _days = timedelta(days=int(self.config.Timedelta))
            while self.d._since < self.d._until:
                self.config.Since = str(self.d._until - _days)
                self.config.Until = str(self.d._until)
                if len(self.feed) > 0:
                    await self.tweets()
                else:
                    self.d._until = self.d._until - _days
                    self.feed = [-1]

                if get.Limit(self.config.Limit, self.count):
                    self.d._until = self.d._until - _days
                    self.feed = [-1]
        else:
            while True:
                if len(self.feed) > 0:
                    if self.config.Followers or self.config.Following:
                        await self.follow()
                    elif self.config.Favorites:
                        await self.favorite()
                    elif self.config.Profile:
                        await self.profile()
                    elif self.config.TwitterSearch:
                        await self.tweets()
                else:
                    break
            
                if get.Limit(self.config.Limit, self.count):
                    break
        
        if self.config.Count:
            verbose.Count(self.count, self.config.Username)

def run(config):
    get_event_loop().run_until_complete(Twint(config).main())

def Favorites(config):
    config.Favorites = True
    run(config)

def Followers(config):
    config.Followers = True
    run(config)

def Following(config):
    config.Following = True
    run(config)

def Profile(config):
    config.Profile = True
    run(config)

def Search(config):
    config.TwitterSearch = True
    run(config)



from datetime import datetime
import sqlite3
import sys

def Conn(database):
    if database:
        print("[+] Inserting into Database: " + str(database))
        conn = init(database)
        if isinstance(conn, str):
            print(str)
            sys.exit(1)
    else:
        conn = ""

    return conn

def init(db):
    try:
        conn = sqlite3.connect(db)
        cursor = conn.cursor()
        table_tweets = """
            CREATE TABLE IF NOT EXISTS
                tweets (
                    id integer not null,
                    user_id integer,
                    date text not null,
                    time text not null,
                    timezone text not null,
                    location text not null,
                    user text not null,
                    tweet text not null,
                    replies integer,
                    likes integer,
                    retweets integer,
                    hashtags text,
                    link text,
                    retweet bool,
                    user_rt text,
                    mentions text,
                    date_update text not null,
                    PRIMARY KEY (id)
                );
        """
        cursor.execute(table_tweets)

        table_followers_names = """
            CREATE TABLE IF NOT EXISTS
                followers_names (
                    user text not null,
                    date_update text not null,
                    follower text not null,
                    PRIMARY KEY (user, follower)
                );

        """
        cursor.execute(table_followers_names)

        table_following_names = """
            CREATE TABLE IF NOT EXISTS
                following_names (
                    user text not null,
                    date_update text not null,
                    follows text not null,
                    PRIMARY KEY (user, follows)
                );
        """
        cursor.execute(table_following_names)

        table_followers = """
            CREATE TABLE IF NOT EXISTS
                followers (
                    id integer not null,
                    name text,
                    username text not null,
                    bio text,
                    url text,
                    join_date text not null,
                    join_time text not null,
                    tweets integer,
                    following integer,
                    followers integer,
                    likes integer,
                    media integer,
                    private text not null,
                    verified text not null,
                    avatar text not null,
                    date_update text not null,
                    follower text not null,
                    PRIMARY KEY (id, username, follower)
                );
        """
        cursor.execute(table_followers)

        table_following = """
            CREATE TABLE IF NOT EXISTS
                following (
                    id integer not null,
                    name text,
                    username text not null,
                    bio text,
                    location text,
                    url text,
                    join_date text not null,
                    join_time text not null,
                    tweets integer,
                    following integer,
                    followers integer,
                    likes integer,
                    media integer,
                    private text not null,
                    verified text not null,
                    avatar text not null,
                    date_update text not null,
                    follows text not null,
                    PRIMARY KEY (id, username, follows)
                );
        """
        cursor.execute(table_following)

        return conn
    except Exception as e:
        return str(e)

def fTable(Followers):
    if Followers:
        table = "followers_names"
    else:
        table = "following_names"

    return table

def uTable(Followers):
    if Followers:
        table = "followers"
    else:
        table = "following"

    return table

def follow(conn, Username, Followers, User):
    try:
        date_time = str(datetime.now())
        cursor = conn.cursor()
        entry = (User, date_time, Username,)
        table = fTable(Followers)
        query = f"INSERT INTO {table} VALUES(?,?,?)"
        cursor.execute(query, entry)
        conn.commit()
    except sqlite3.IntegrityError:
        pass

def user(conn, Username, Followers,  User):
    try:
        date_time = str(datetime.now())
        cursor = conn.cursor()
        entry = (User.id,
                User.name,
                User.username,
                User.bio,
                User.location,
                User.url,
                User.join_date,
                User.join_time,
                User.tweets,
                User.following,
                User.followers,
                User.likes,
                User.media_count,
                User.is_private,
                User.is_verified,
                User.avatar,
                date_time,
                Username,)
        table = uTable(Followers)
        query = f"INSERT INTO {table} VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)"
        cursor.execute(query, entry)
        conn.commit()
    except sqlite3.IntegrityError:
        pass

def tweets(conn, Tweet, config):
    try:
        date_time = str(datetime.now())
        cursor = conn.cursor()
        entry = (Tweet.id,
                    Tweet.user_id,
                    Tweet.datestamp,
                    Tweet.timestamp,
                    Tweet.timezone,
                    Tweet.location,
                    Tweet.username,
                    Tweet.tweet,
                    Tweet.replies,
                    Tweet.likes,
                    Tweet.retweets,
                    ",".join(Tweet.hashtags),
                    Tweet.link,
                    Tweet.retweet,
                    Tweet.user_rt,
                    ",".join(Tweet.mentions),
                    date_time)
        cursor.execute('INSERT INTO tweets VALUES(?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?,?)', entry)
        conn.commit()
    except sqlite3.IntegrityError:
        pass



def tweetData(t):
    data = {
            "id": int(t.id),
            "date": t.datestamp,
            "time": t.timestamp,
            "timezone": t.timezone,
            "user_id": int(t.user_id),
            "username": t.username,
            "tweet": t.tweet,
            "replies": int(t.replies),
            "retweets": int(t.retweets),
            "likes": int(t.likes),
            "location": t.location,
            "hashtags": ",".join(t.hashtags),
            "link": t.link,
            "retweet": t.retweet,
            "user_rt": t.user_rt,
            "mentions": ",".join(t.mentions)
            }
    return data

def tweetFieldnames():
    fieldnames = [
            "id",
            "date",
            "time",
            "timezone",
            "user_id",
            "username",
            "tweet",
            "replies",
            "retweets",
            "likes",
            "location",
            "hashtags",
            "link",
            "retweet",
            "user_rt",
            "mentions"
            ]
    return fieldnames

def userData(u):
    data = {
            "id": int(u.id),
            "name": u.name,
            "username": u.username,
            "bio": u.bio,
            "location": u.location,
            "url": u.url,
            "join_date": u.join_date,
            "join_time": u.join_time,
            "tweets": int(u.tweets),
            "following": int(u.following),
            "followers": int(u.followers),
            "likes": int(u.likes),
            "media": int(u.media_count),
            "private": u.is_private,
            "verified": u.is_verified,
            "avatar": u.avatar
            }
    return data

def userFieldnames():
    fieldnames = [
            "id",
            "name",
            "username",
            "bio",
            "location",
            "url",
            "join_date",
            "join_time",
            "tweets",
            "following",
            "followers",
            "likes",
            "media",
            "private",
            "verified",
            "avatar"
            ]
    return fieldnames

def usernameData(u):
    return {"username": u}

def usernameFieldnames():
    return ["username"]

def Data(obj, _type):
    if _type == "user":
        ret = userData(obj)
    elif _type == "username":
        ret = usernameData(obj)
    else:
        ret = tweetData(obj)

    return ret

def Fieldnames(_type):
    if _type == "user":
        ret = userFieldnames()
    elif _type == "username":
        ret = usernameFieldnames()
    else:
        ret = tweetFieldnames()

    return ret
